@@ -1,4 +1,5 @@
 from pymaster import nmtlib as lib
+from pymaster.utils import _toeplitz_sanity
 import numpy as np
 
 
@@ -33,7 +34,9 @@ class NmtCovarianceWorkspace(object):
 
     def compute_coupling_coefficients(self, fla1, fla2,
                                       flb1=None, flb2=None,
-                                      lmax=None, n_iter=3):
+                                      lmax=None, n_iter=3,
+                                      l_toeplitz=-1,
+                                      l_exact=-1, dl_band=-1):
         """
         Computes coupling coefficients of the Gaussian covariance \
         between the power spectra of two pairs of NmtField objects \
@@ -49,6 +52,16 @@ class NmtCovarianceWorkspace(object):
             power spectrum whose covariance you want to compute. If \
             None, fla1,fla2 will be used.
         :param n_iter: number of iterations when computing a_lms.
+        :param l_toeplitz: if a positive number, the Toeplitz approximation \
+            described in Louis et al. 2020 (arXiv:2010.14344) will be used. \
+            In that case, this quantity corresponds to ell_toeplitz in Fig. \
+            3 of that paper.
+        :param l_exact: if `l_toeplitz>0`, this quantity corresponds to \
+            ell_exact in Fig. 3 of Louis et al. 2020.  Ignored if \
+            `l_toeplitz<=0`.
+        :param dl_band: if `l_toeplitz>0`, this quantity corresponds to \
+            Delta ell_band in Fig. 3 of Louis et al. 2020.  Ignored if \
+            `l_toeplitz<=0`.
         """
         if flb1 is None:
             flb1 = fla1
@@ -67,8 +80,10 @@ class NmtCovarianceWorkspace(object):
         if lmax is None:
             lmax = lib.get_lmax_from_cs_py(fla1.fl.cs)
 
+        _toeplitz_sanity(l_toeplitz, l_exact, dl_band, lmax, fla1, flb1)
         self.wsp = lib.covar_workspace_init_py(fla1.fl, fla2.fl, flb1.fl,
-                                               flb2.fl, lmax, n_iter)
+                                               flb2.fl, lmax, n_iter,
+                                               l_toeplitz, l_exact, dl_band)
 
     def write_to(self, fname):
         """
