@@ -54,6 +54,24 @@ class TestFieldCAR(unittest.TestCase):
             self.tmp[i][1, :, :] = -np.sqrt(15./2./np.pi)*sth**2/4.
             self.tmp[i][2, :, :] = -np.sqrt(105./2./np.pi)*cth*sth**2/2.
 
+    def test_field_lite(self):
+        # Lite field
+        fl = nmt.NmtField(self.msk, [self.mps[0]], wcs=self.wcs,
+                          beam=self.beam, lite=True)
+        # Empty field
+        with self.assertRaises(ValueError):  # No maps and no spin
+            fe = nmt.NmtField(self.msk, None, wcs=self.wcs,
+                              beam=self.beam)
+        fe = nmt.NmtField(self.msk, None, wcs=self.wcs,
+                          beam=self.beam, spin=1)
+
+        # Error checks
+        for f in [fl, fe]:
+            with self.assertRaises(ValueError):  # Query maps
+                f.get_maps()
+            with self.assertRaises(ValueError):  # Query templates
+                f.get_templates()
+
     def test_field_alloc(self):
         # No templates
         f0 = nmt.NmtField(self.msk, [self.mps[0]],
@@ -275,6 +293,22 @@ class TestFieldHPX(unittest.TestCase):
         self.assertEqual(len(f0.get_templates()), 5)
         self.assertEqual(len(f2.get_templates()), 5)
 
+    def test_field_lite(self):
+        # Lite field
+        fl = nmt.NmtField(self.msk, [self.mps[0]],
+                          beam=self.beam, lite=True)
+        # Empty field
+        with self.assertRaises(ValueError):  # No maps and no spin
+            fe = nmt.NmtField(self.msk, None, beam=self.beam)
+        fe = nmt.NmtField(self.msk, None, beam=self.beam, spin=1)
+
+        # Error checks
+        for f in [fl, fe]:
+            with self.assertRaises(ValueError):  # Query maps
+                f.get_maps()
+            with self.assertRaises(ValueError):  # Query templates
+                f.get_templates()
+
     def test_field_error(self):
         with self.assertRaises(ValueError):  # Incorrect mask size
             nmt.NmtField(self.msk[:15], self.mps)
@@ -296,6 +330,21 @@ class TestFieldHPX(unittest.TestCase):
         with self.assertRaises(ValueError):  # Passing crap as beam
             nmt.NmtField(self.msk, [self.mps[0]], beam=1)
 
+        # Automatically assign spin = 0 for a single map
+        f = nmt.NmtField(self.msk, [self.mps[0]], n_iter=0)
+        self.assertTrue(f.fl.spin == 0)
+        # Automatically assign spin = 2 for 2 maps
+        f = nmt.NmtField(self.msk, [self.mps[1], self.mps[2]], n_iter=0)
+        self.assertTrue(f.fl.spin == 2)
+        with self.assertRaises(ValueError):  # Spin=0 but 2 maps
+            f = nmt.NmtField(self.msk, [self.mps[1], self.mps[2]],
+                             spin=0, n_iter=0)
+        with self.assertRaises(ValueError):  # Spin=1 but 1 maps
+            f = nmt.NmtField(self.msk, [self.mps[0]], spin=1, n_iter=0)
+        with self.assertRaises(ValueError):
+            f = nmt.NmtField(self.msk, [self.mps[1], self.mps[2]], spin=1,
+                             purify_b=True, n_iter=0)
+
 
 class TestFieldFsk(unittest.TestCase):
     def setUp(self):
@@ -375,6 +424,25 @@ class TestFieldFsk(unittest.TestCase):
         self.assertTrue(np.all(np.fabs(c22-c22_msk) /
                                np.mean(c22) < 1E-10))
 
+    def test_field_lite(self):
+        # Lite field
+        fl = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                              [self.mps[0]], beam=self.beam,
+                              lite=True)
+        # Empty field
+        with self.assertRaises(ValueError):  # No maps and no spin
+            fe = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                                  None, beam=self.beam)
+        fe = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                              None, beam=self.beam, spin=1)
+
+        # Error checks
+        for f in [fl, fe]:
+            with self.assertRaises(ValueError):  # Query maps
+                f.get_maps()
+            with self.assertRaises(ValueError):  # Query templates
+                f.get_templates()
+
     def test_field_flat_alloc(self):
         # No templates
         f0 = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
@@ -437,6 +505,25 @@ class TestFieldFsk(unittest.TestCase):
         with self.assertRaises(ValueError):  # Passing crap beam
             nmt.NmtFieldFlat(self.lx, self.ly, self.msk, [self.mps[0]], beam=1)
 
+        # Automatically assign spin = 0 for a single map
+        f = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                             [self.mps[0]])
+        self.assertTrue(f.fl.spin == 0)
+        # Automatically assign spin = 2 for 2 maps
+        f = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                             [self.mps[1], self.mps[2]])
+        self.assertTrue(f.fl.spin == 2)
+        with self.assertRaises(ValueError):  # Spin=0 but 2 maps
+            f = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                                 [self.mps[1], self.mps[2]], spin=0)
+        with self.assertRaises(ValueError):  # Spin=1 but 1 maps
+            f = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                                 [self.mps[0]], spin=1)
+        with self.assertRaises(ValueError):
+            f = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                                 [self.mps[1], self.mps[2]], spin=1,
+                                 purify_b=True)
+
 
 if __name__ == '__main__':
     unittest.main()
